@@ -1,22 +1,22 @@
+{-# LANGUAGE BangPatterns              #-}
+{-# LANGUAGE CPP                       #-}
+{-# LANGUAGE EmptyDataDecls            #-}
+{-# LANGUAGE ExistentialQuantification #-}
+{-# LANGUAGE FlexibleInstances         #-}
+{-# LANGUAGE ForeignFunctionInterface  #-}
+{-# LANGUAGE OverloadedStrings         #-}
+{-# LANGUAGE Rank2Types                #-}
+{-# LANGUAGE TypeSynonymInstances      #-}
+
+------------------------------------------------------------------------------
 -- | An internal Snap module containing HTTP types.
 --
 -- /N.B./ this is an internal interface, please don't write user code that
 -- depends on it. Most of these declarations (except for the
 -- unsafe/encapsulation-breaking ones) are re-exported from "Snap.Core".
-
-{-# LANGUAGE BangPatterns #-}
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE EmptyDataDecls #-}
-{-# LANGUAGE ForeignFunctionInterface #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE Rank2Types #-}
-{-# LANGUAGE TypeSynonymInstances #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE ExistentialQuantification #-}
-
+--
 module Snap.Internal.Http.Types where
 
-
 ------------------------------------------------------------------------------
 import           Blaze.ByteString.Builder
 import           Control.Monad (liftM)
@@ -24,6 +24,8 @@ import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as B
 import           Data.ByteString.Internal (c2w,w2c)
 import qualified Data.ByteString as S
+import           Data.CaseInsensitive   (CI)
+import qualified Data.CaseInsensitive as CI
 import           Data.Int
 import qualified Data.IntMap as IM
 import           Data.IORef
@@ -36,7 +38,7 @@ import           Data.Time.Clock
 import           Foreign.C.Types
 import           Prelude hiding (take)
 
-
+------------------------------------------------------------------------------
 #ifdef PORTABLE
 import           Data.Time.Format
 import           Data.Time.LocalTime
@@ -50,18 +52,16 @@ import           Foreign.C.String
 #endif
 
 ------------------------------------------------------------------------------
-import           Data.CaseInsensitive   (CI)
-import qualified Data.CaseInsensitive as CI
 import           Snap.Iteratee (Enumerator)
 import qualified Snap.Iteratee as I
 import           Snap.Types.Headers (Headers)
 import qualified Snap.Types.Headers as H
 
+
 #ifndef PORTABLE
 
 ------------------------------------------------------------------------------
 -- foreign imports from cbits
-
 foreign import ccall unsafe "set_c_locale"
         set_c_locale :: IO ()
 
@@ -80,7 +80,6 @@ foreign import ccall unsafe "c_format_log_time"
 ------------------------------------------------------------------------------
 -- | A typeclass for datatypes which contain HTTP headers.
 class HasHeaders a where
-
     -- | Modify the datatype's headers.
     updateHeaders :: (Headers -> Headers) -> a -> a
 
@@ -113,7 +112,7 @@ getHeaders k a = H.lookup k $ headers a
 -- | Gets a header value out of a 'HasHeaders' datatype. If many headers came
 -- in with the same name, they will be catenated together.
 getHeader :: (HasHeaders a) => CI ByteString -> a -> Maybe ByteString
-getHeader k a = liftM (S.intercalate " ") (H.lookup k $ headers a)
+getHeader k a = liftM (S.intercalate ",") (H.lookup k $ headers a)
 
 
 ------------------------------------------------------------------------------
@@ -187,30 +186,30 @@ newtype SomeEnumerator = SomeEnumerator (forall a . Enumerator ByteString IO a)
 data Request = Request
     { -- | The server name of the request, as it came in from the request's
       -- @Host:@ header.
-      rqServerName     :: !ByteString
+      rqServerName     :: ByteString
 
       -- | Returns the port number the HTTP server is listening on.
     , rqServerPort     :: !Int
 
       -- | The remote IP address.
-    , rqRemoteAddr     :: !ByteString
+    , rqRemoteAddr     :: ByteString
 
       -- | The remote TCP port number.
-    , rqRemotePort     :: !Int
+    , rqRemotePort     :: Int
 
       -- | The local IP address for this request.
-    , rqLocalAddr      :: !ByteString
+    , rqLocalAddr      :: ByteString
 
       -- | Returns the port number the HTTP server is listening on.
-    , rqLocalPort      :: !Int
+    , rqLocalPort      :: Int
 
       -- | Returns the HTTP server's idea of its local hostname.
-    , rqLocalHostname  :: !ByteString
+    , rqLocalHostname  :: ByteString
 
       -- | Returns @True@ if this is an @HTTPS@ session.
-    , rqIsSecure       :: !Bool
+    , rqIsSecure       :: Bool
     , rqHeaders        :: Headers
-    , rqBody           :: IORef SomeEnumerator
+    , rqBody           :: !(IORef SomeEnumerator)
 
       -- | Returns the @Content-Length@ of the HTTP request body.
     , rqContentLength  :: !(Maybe Int)
@@ -219,38 +218,26 @@ data Request = Request
     , rqMethod         :: !Method
 
       -- | Returns the HTTP version used by the client.
-    , rqVersion        :: !HttpVersion
+    , rqVersion        :: HttpVersion
 
       -- | Returns a list of the cookies that came in from the HTTP request
       -- headers.
     , rqCookies        :: [Cookie]
 
-
-      -- | We'll be doing web components (or \"snaplets\") for version 0.2.
-      -- The \"snaplet path\" refers to the place on the URL where your
-      -- containing snaplet is hung. The value of 'rqSnapletPath' is either
-      -- @\"\"@ (at the top-level context) or is a path beginning with a
-      -- slash, but not ending with one.
+      -- | Handlers can be hung on a @URI@ \"entry point\"; this is called the
+      -- \"context path\". If a handler is hung on the context path
+      -- @\"\/foo\/\"@, and you request @\"\/foo\/bar\"@, the value of
+      -- 'rqPathInfo' will be @\"bar\"@.
       --
-      -- An identity is that:
+      -- The following identity holds:
       --
-      -- > rqURI r == S.concat [ rqSnapletPath r
-      -- >                     , rqContextPath r
+      -- > rqURI r == S.concat [ rqContextPath r
       -- >                     , rqPathInfo r
       -- >                     , let q = rqQueryString r
       -- >                       in if S.null q
       -- >                            then ""
       -- >                            else S.append "?" q
       -- >                     ]
-      --
-      -- note that until we introduce snaplets in v0.2, 'rqSnapletPath' will
-      -- be \"\"
-    , rqSnapletPath    :: !ByteString
-
-      -- | Handlers can (/will be; --ed/) be hung on a @URI@ \"entry point\";
-      -- this is called the \"context path\". If a handler is hung on the
-      -- context path @\"\/foo\/\"@, and you request @\"\/foo\/bar\"@, the
-      -- value of 'rqPathInfo' will be @\"bar\"@.
     , rqPathInfo       :: !ByteString
 
       -- | The \"context path\" of the request; catenating 'rqContextPath',
@@ -266,10 +253,19 @@ data Request = Request
       -- | Returns the HTTP query string for this 'Request'.
     , rqQueryString    :: !ByteString
 
-      -- | Returns the 'Params' mapping for this 'Request'. \"Parameters\" are
-      -- automatically decoded from the query string and @POST@ body and
-      -- entered into this mapping.
+      -- | Returns the parameters mapping for this 'Request'. \"Parameters\"
+      -- are automatically decoded from the URI's query string and @POST@ body
+      -- and entered into this mapping. The 'rqParams' value is thus a union of
+      -- 'rqQueryParams' and 'rqPostParams'.
     , rqParams         :: Params
+
+      -- | The parameter mapping decoded from the URI's query string.
+    , rqQueryParams    :: Params
+
+      -- | The parameter mapping decoded from the POST body. Note that Snap
+      -- only auto-decodes POST request bodies when the request's
+      -- @Content-Type@ is @application/x-www-form-urlencoded@.
+    , rqPostParams     :: Params
     }
 
 
@@ -292,7 +288,6 @@ instance Show Request where
                     , cookies
                     , pathinfo
                     , contextpath
-                    , snapletpath
                     , uri
                     , params
                     ]
@@ -333,7 +328,6 @@ instance Show Request where
           ]
       pathinfo      = concat [ "pathinfo: ", toStr $ rqPathInfo r ]
       contextpath   = concat [ "contextpath: ", toStr $ rqContextPath r ]
-      snapletpath   = concat [ "snapletpath: ", toStr $ rqSnapletPath r ]
       uri           = concat [ "URI: ", toStr $ rqURI r ]
       params'       = "      " ++
                       (concat $ intersperse "\n " $
@@ -410,6 +404,12 @@ data Response = Response
       -- | If true, we are transforming the request body with
       -- 'transformRequestBody'
     , rspTransformingRqBody :: !Bool
+
+      -- | Controls whether Snap will buffer the output or not. You may wish to
+      -- disable buffering when using Comet-like techniques which rely on the
+      -- immediate sending of output data in order to maintain interactive
+      -- semantics.
+    , rspOutputBuffering    :: !Bool
     }
 
 
@@ -455,6 +455,26 @@ rqParam k rq = Map.lookup k $ rqParams rq
 {-# INLINE rqParam #-}
 
 
+------------------------------------------------------------------------------
+-- | Looks up the value(s) for the given named parameter in the POST parameters
+-- mapping.
+rqPostParam :: ByteString           -- ^ parameter name to look up
+            -> Request              -- ^ HTTP request
+            -> Maybe [ByteString]
+rqPostParam k rq = Map.lookup k $ rqPostParams rq
+{-# INLINE rqPostParam #-}
+
+
+------------------------------------------------------------------------------
+-- | Looks up the value(s) for the given named parameter in the query
+-- parameters mapping.
+rqQueryParam :: ByteString           -- ^ parameter name to look up
+             -> Request              -- ^ HTTP request
+             -> Maybe [ByteString]
+rqQueryParam k rq = Map.lookup k $ rqQueryParams rq
+{-# INLINE rqQueryParam #-}
+
+
 ------------------------------------------------------------------------------
 -- | Modifies the parameters mapping (which is a @Map ByteString ByteString@)
 -- in a 'Request' using the given function.
@@ -475,15 +495,17 @@ rqSetParam :: ByteString        -- ^ parameter name
 rqSetParam k v = rqModifyParams $ Map.insert k v
 {-# INLINE rqSetParam #-}
 
-------------------------------------------------------------------------------
--- responses
-------------------------------------------------------------------------------
 
+                                ---------------
+                                -- responses --
+                                ---------------
+
+------------------------------------------------------------------------------
 -- | An empty 'Response'.
 emptyResponse :: Response
 emptyResponse = Response H.empty Map.empty (1,1) Nothing
                          (Enum (I.enumBuilder mempty))
-                         200 "OK" False
+                         200 "OK" False True
 
 
 ------------------------------------------------------------------------------
@@ -614,19 +636,49 @@ clearContentLength r = r { rspContentLength = Nothing }
 
 
 ------------------------------------------------------------------------------
--- HTTP dates
+-- | The buffering mode controls whether Snap will buffer the output or not.
+-- You may wish to disable buffering when using Comet-like techniques which
+-- rely on the immediate sending of output data in order to maintain
+-- interactive semantics.
+getBufferingMode :: Response -> Bool
+getBufferingMode = rspOutputBuffering
+{-# INLINE getBufferingMode #-}
 
+
+------------------------------------------------------------------------------
+-- | The buffering mode controls whether Snap will buffer the output or not.
+-- You may wish to disable buffering when using Comet-like techniques which
+-- rely on the immediate sending of output data in order to maintain
+-- interactive semantics.
+setBufferingMode :: Bool        -- ^ if True, buffer the output, if False, send
+                                -- output immediately
+                 -> Response
+                 -> Response
+setBufferingMode b r = r { rspOutputBuffering = b }
+{-# INLINE setBufferingMode #-}
+
+
+                               ----------------
+                               -- HTTP dates --
+                               ----------------
+
+------------------------------------------------------------------------------
 -- | Converts a 'CTime' into an HTTP timestamp.
 formatHttpTime :: CTime -> IO ByteString
 
+
+------------------------------------------------------------------------------
 -- | Converts a 'CTime' into common log entry format.
 formatLogTime :: CTime -> IO ByteString
 
+
+------------------------------------------------------------------------------
 -- | Converts an HTTP timestamp into a 'CTime'.
 parseHttpTime :: ByteString -> IO CTime
 
 #ifdef PORTABLE
 
+------------------------------------------------------------------------------
 formatHttpTime = return . format . toUTCTime
   where
     format :: UTCTime -> ByteString
@@ -635,9 +687,11 @@ formatHttpTime = return . format . toUTCTime
     toUTCTime :: CTime -> UTCTime
     toUTCTime = posixSecondsToUTCTime . realToFrac
 
+
+------------------------------------------------------------------------------
 formatLogTime ctime = do
   t <- utcToLocalZonedTime $ toUTCTime ctime
-  return $ format t
+  return $! format t
 
   where
     format :: ZonedTime -> ByteString
@@ -647,6 +701,7 @@ formatLogTime ctime = do
     toUTCTime = posixSecondsToUTCTime . realToFrac
 
 
+------------------------------------------------------------------------------
 parseHttpTime = return . toCTime . prs . toStr
   where
     prs :: String -> Maybe UTCTime
@@ -658,16 +713,21 @@ parseHttpTime = return . toCTime . prs . toStr
 
 #else
 
+------------------------------------------------------------------------------
 formatLogTime t = do
     ptr <- mallocBytes 40
     c_format_log_time t ptr
     S.unsafePackMallocCString ptr
 
+
+------------------------------------------------------------------------------
 formatHttpTime t = do
     ptr <- mallocBytes 40
     c_format_http_time t ptr
     S.unsafePackMallocCString ptr
 
+
+------------------------------------------------------------------------------
 parseHttpTime s = S.unsafeUseAsCString s $ \ptr ->
     c_parse_http_time ptr
 
