@@ -4,43 +4,45 @@
 
 module Snap.Internal.Test.RequestBuilder
   ( RequestBuilder
-  , buildRequest
   , MultipartParams
   , MultipartParam(..)
   , FileData      (..)
   , RequestType   (..)
-  , setQueryStringRaw
-  , setQueryString
-  , setRequestType
   , addHeader
-  , setHeader
-  , setContentType
-  , setSecure
-  , setHttpVersion
-  , setRequestPath
-  , get
-  , postUrlEncoded
-  , postMultipart
-  , put
-  , postRaw
+  , buildRequest
   , delete
-  , runHandler
-  , runHandler'
   , dumpResponse
+  , evalHandler
+  , evalHandlerM
+  , get
+  , postMultipart
+  , postRaw
+  , postUrlEncoded
+  , put
   , responseToString
+  , runHandler
+  , runHandlerM
+  , setContentType
+  , setHeader
+  , setHttpVersion
+  , setQueryString
+  , setQueryStringRaw
+  , setRequestPath
+  , setRequestType
+  , setSecure
   ) where
 
 ------------------------------------------------------------------------------
 import           Blaze.ByteString.Builder
 import           Blaze.ByteString.Builder.Char8
-import           Control.Monad.State hiding (get, put)
-import qualified Control.Monad.State as State
-import qualified Data.ByteString.Base16   as B16
-import           Data.ByteString.Char8 (ByteString)
-import qualified Data.ByteString.Char8 as S
-import           Data.CaseInsensitive   (CI)
+import           Control.Monad.State            hiding (get, put)
+import qualified Control.Monad.State            as State
+import qualified Data.ByteString.Base16         as B16
+import           Data.ByteString.Char8          (ByteString)
+import qualified Data.ByteString.Char8          as S
+import           Data.CaseInsensitive           (CI)
 import           Data.IORef
-import qualified Data.Map as Map
+import qualified Data.Map                       as Map
 import           Data.Monoid
 import           Data.Word
 import           System.PosixCompat.Time
@@ -49,17 +51,21 @@ import           System.Random.MWC
 import           Snap.Internal.Http.Types hiding (addHeader,
                                                   setContentType,
                                                   setHeader)
-import qualified Snap.Internal.Http.Types as H
+import qualified Snap.Internal.Http.Types       as H
 import           Snap.Internal.Parsing
-import           Snap.Iteratee hiding (map)
-import           Snap.Core hiding (addHeader, setContentType, setHeader)
-import qualified Snap.Types.Headers as H
+import           Snap.Internal.Types            (evalSnap)
+import           Snap.Iteratee                  hiding (map)
+import           Snap.Core                      hiding ( addHeader
+                                                       , setContentType
+                                                       , setHeader )
+import qualified Snap.Types.Headers             as H
+
 
 ------------------------------------------------------------------------------
 -- | RequestBuilder is a monad transformer that allows you to conveniently
 -- build a snap 'Request' for testing.
 newtype RequestBuilder m a = RequestBuilder (StateT Request m a)
-  deriving (Monad, MonadIO, MonadTrans)
+  deriving (Monad, MonadIO, MonadState Request, MonadTrans)
 
 
 ------------------------------------------------------------------------------
@@ -81,15 +87,20 @@ mkDefaultRequest = do
                      (1,1)
                      []
                      ""
-                     ""
                      "/"
                      "/"
                      ""
                      Map.empty
+                     Map.empty
+                     Map.empty
 
 
 ------------------------------------------------------------------------------
 -- | Runs a 'RequestBuilder', producing the desired 'Request'.
+--
+-- N.B. /please/ don't use the request you get here in a real Snap application;
+-- things will probably break. Don't say you weren't warned :-)
+--
 buildRequest :: MonadIO m => RequestBuilder m () -> m Request
 buildRequest mm = do
     let (RequestBuilder m) = (mm >> fixup)
@@ -123,18 +134,20 @@ buildRequest mm = do
 
     fixupParams = do
         rq <- rGet
-        let q   = rqQueryString rq
-        let pms = parseUrlEncoded q
+        let query       = rqQueryString rq
+        let queryParams = parseUrlEncoded query
+        let mbCT        = getHeader "Content-Type" rq
 
-        let mbCT = getHeader "Content-Type" rq
-        post <- if mbCT == Just "application/x-www-form-urlencoded"
-                  then do
-                    (SomeEnumerator e) <- liftIO $ readIORef $ rqBody rq
-                    s <- liftM S.concat (liftIO $ run_ $ e $$ consume)
-                    return $ parseUrlEncoded s
-                  else return Map.empty
+        postParams <- if mbCT == Just "application/x-www-form-urlencoded"
+                        then do
+                          (SomeEnumerator e) <- liftIO $ readIORef $ rqBody rq
+                          s <- liftM S.concat (liftIO $ run_ $ e $$ consume)
+                          return $ parseUrlEncoded s
+                        else return Map.empty
+
+        rPut $ rq { rqParams      = Map.unionWith (++) queryParams postParams
+                  , rqQueryParams = queryParams }
 
-        rPut $ rq { rqParams = Map.unionWith (++) pms post }
 
 ------------------------------------------------------------------------------
 -- | A request body of type \"@multipart/form-data@\" consists of a set of
@@ -262,12 +275,15 @@ encodeFormData boundary name vals =
                          , fromByteString b
                          , fromByteString "--\r\n--" ]
 
+
+------------------------------------------------------------------------------
 multipartMixed :: ByteString -> Builder
 multipartMixed b = mconcat [ fromByteString "Content-Type: multipart/mixed"
                            , fromByteString "; boundary="
                            , fromByteString b
                            , fromByteString "\r\n" ]
 
+
 ------------------------------------------------------------------------------
 encodeFiles :: ByteString -> ByteString -> [FileData] -> IO Builder
 encodeFiles boundary name files =
@@ -285,6 +301,7 @@ encodeFiles boundary name files =
                            ]
 
   where
+    --------------------------------------------------------------------------
     contentDisposition fn = mconcat [
                               fromByteString "Content-Disposition: attachment"
                             , fromByteString "; filename=\""
@@ -292,12 +309,14 @@ encodeFiles boundary name files =
                             , fromByteString "\"\r\n"
                             ]
 
+    --------------------------------------------------------------------------
     contentType ct = mconcat [
                        fromByteString "Content-Type: "
                      , fromByteString ct
                      , cr
                      ]
 
+    --------------------------------------------------------------------------
     oneVal b (FileData fileName ct contents) =
         mconcat [ fromByteString b
                 , cr
@@ -309,8 +328,9 @@ encodeFiles boundary name files =
                 , fromByteString "\r\n--"
                 ]
 
+    --------------------------------------------------------------------------
     hdr = multipartHeader boundary name
-    cr = fromByteString "\r\n"
+    cr  = fromByteString "\r\n"
 
 
 ------------------------------------------------------------------------------
@@ -347,8 +367,7 @@ encodeMultipart kvps = do
 fixupURI :: Monad m => RequestBuilder m ()
 fixupURI = do
     rq <- rGet
-    let u = S.concat [ rqSnapletPath rq
-                     , rqContextPath rq
+    let u = S.concat [ rqContextPath rq
                      , rqPathInfo rq
                      , let q = rqQueryString rq
                        in if S.null q
@@ -414,12 +433,14 @@ setHttpVersion v = rModify $ \rq -> rq { rqVersion = v }
 -- in your test request, you must use 'setQueryString' or 'setQueryStringRaw'.
 -- Note that 'rqContextPath' is never set by any 'RequestBuilder' function.
 setRequestPath :: Monad m => ByteString -> RequestBuilder m ()
-setRequestPath p = do
-    rModify $ \rq -> rq { rqSnapletPath = ""
-                        , rqContextPath = ""
+setRequestPath p0 = do
+    rModify $ \rq -> rq { rqContextPath = "/"
                         , rqPathInfo    = p }
     fixupURI
 
+  where
+    p = if S.isPrefixOf "/" p0 then S.drop 1 p0 else p0
+
 
 ------------------------------------------------------------------------------
 -- | Builds an HTTP \"GET\" request with the given query parameters.
@@ -496,32 +517,15 @@ postRaw uri contentType postData = do
     setRequestPath uri
 
 
-------------------------------------------------------------------------------
--- | Given a web handler in some 'MonadSnap' monad, and a 'RequestBuilder'
--- defining a test request, runs the handler, producing an HTTP 'Response'.
-runHandler' :: (MonadIO m, MonadSnap n) =>
-               (forall a . Request -> n a -> m Response)
-            -- ^ a function defining how the 'MonadSnap' monad should be run
-            -> RequestBuilder m ()
-            -- ^ a request builder
-            -> n b
-            -- ^ a web handler
-            -> m Response
-runHandler' rSnap rBuilder snap = do
-    rq  <- buildRequest rBuilder
-    rsp <- rSnap rq snap
-    t1  <- liftIO (epochTime >>= formatHttpTime)
-    return $ H.setHeader "Date" t1 rsp
-
-
 ------------------------------------------------------------------------------
 -- | Given a web handler in the 'Snap' monad, and a 'RequestBuilder' defining
 -- a test request, runs the handler, producing an HTTP 'Response'.
+--
 runHandler :: MonadIO m =>
               RequestBuilder m ()   -- ^ a request builder
            -> Snap a                -- ^ a web handler
            -> m Response
-runHandler = runHandler' rs
+runHandler = runHandlerM rs
   where
     rs rq s = do
         (_,rsp) <- liftIO $ run_ $ runSnap s
@@ -531,6 +535,66 @@ runHandler = runHandler' rs
         return rsp
 
 
+------------------------------------------------------------------------------
+-- | Given a web handler in some arbitrary 'MonadSnap' monad, a function
+-- specifying how to evaluate it within the context of the test monad, and a
+-- 'RequestBuilder' defining a test request, runs the handler, producing an
+-- HTTP 'Response'.
+runHandlerM :: (MonadIO m, MonadSnap n) =>
+               (forall a . Request -> n a -> m Response)
+            -- ^ a function defining how the 'MonadSnap' monad should be run
+            -> RequestBuilder m ()
+            -- ^ a request builder
+            -> n b
+            -- ^ a web handler
+            -> m Response
+runHandlerM rSnap rBuilder snap = do
+    rq  <- buildRequest rBuilder
+    rsp <- rSnap rq snap
+    t1  <- liftIO (epochTime >>= formatHttpTime)
+    return $ H.setHeader "Date" t1 rsp
+
+
+------------------------------------------------------------------------------
+-- | Given a web handler in the 'Snap' monad, and a 'RequestBuilder' defining a
+-- test request, runs the handler and returns the monadic value it produces.
+--
+-- Throws an exception if the 'Snap' handler early-terminates with 'finishWith'
+-- or 'mzero'.
+--
+evalHandler :: MonadIO m =>
+               RequestBuilder m ()
+            -> Snap a
+            -> m a
+evalHandler = evalHandlerM rs
+  where
+    rs rq s = liftIO $ run_
+                     $ evalSnap s (const $ return $! ())
+                                  (const $ return $! ())
+                                  rq
+
+
+------------------------------------------------------------------------------
+-- | Given a web handler in some arbitrary 'MonadSnap' monad, a function
+-- specifying how to evaluate it within the context of the test monad, and a
+-- 'RequestBuilder' defining a test request, runs the handler, returning the
+-- monadic value it produces.
+--
+-- Throws an exception if the 'Snap' handler early-terminates with 'finishWith'
+-- or 'mzero'.
+--
+evalHandlerM :: (MonadIO m, MonadSnap n) =>
+                (forall a . Request -> n a -> m a)  -- ^ a function defining
+                                                    -- how the 'MonadSnap'
+                                                    -- monad should be run
+             -> RequestBuilder m ()                 -- ^ a request builder
+             -> n b                                 -- ^ a web handler
+             -> m b
+evalHandlerM rSnap rBuilder snap = do
+    rq <- buildRequest rBuilder
+    rSnap rq snap
+
+
 ------------------------------------------------------------------------------
 -- | Dumps the given response to stdout.
 dumpResponse :: Response -> IO ()
@@ -547,7 +611,6 @@ responseToString resp = do
     return $ toByteString $ fromShow resp `mappend` b
 
 
-
 ------------------------------------------------------------------------------
 rGet :: Monad m => RequestBuilder m Request
 rGet   = RequestBuilder State.get
