@@ -1,11 +1,9 @@
 {-# LANGUAGE BangPatterns              #-}
 {-# LANGUAGE DeriveDataTypeable        #-}
-{-# LANGUAGE EmptyDataDecls            #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE FlexibleInstances         #-}
 {-# LANGUAGE MultiParamTypeClasses     #-}
 {-# LANGUAGE OverloadedStrings         #-}
-{-# LANGUAGE PackageImports            #-}
 {-# LANGUAGE Rank2Types                #-}
 {-# LANGUAGE ScopedTypeVariables       #-}
 {-# LANGUAGE TypeSynonymInstances      #-}
@@ -34,19 +32,18 @@ import qualified Data.Text as T
 import qualified Data.Text.Lazy as LT
 import           Data.Typeable
 import           Prelude hiding (catch, take)
-
-
-------------------------------------------------------------------
+------------------------------------------------------------------------------
 import           Snap.Internal.Http.Types
+import           Snap.Internal.Exceptions
 import           Snap.Internal.Iteratee.Debug
 import           Snap.Util.Readable
-import           Snap.Iteratee
-
-
-------------------------------------------------------------------------------
--- The Snap Monad
+import           Snap.Iteratee hiding (map)
 ------------------------------------------------------------------------------
 
+
+                             --------------------
+                             -- The Snap Monad --
+                             --------------------
 {-|
 
 'Snap' is the 'Monad' that user web handlers run in. 'Snap' gives you:
@@ -95,8 +92,8 @@ import           Snap.Iteratee
    > a :: Snap ()
    > a = liftIO fireTheMissiles
 
-7. the ability to set a timeout which will kill the handler thread after @N@
-   seconds of inactivity (the default is 20 seconds):
+7. the ability to set or extend a timeout which will kill the handler thread
+   after @N@ seconds of inactivity (the default is 20 seconds):
 
    > a :: Snap ()
    > a = setTimeout 30
@@ -144,10 +141,11 @@ newtype Snap a = Snap {
 
 ------------------------------------------------------------------------------
 data SnapState = SnapState
-    { _snapRequest    :: Request
-    , _snapResponse   :: Response
-    , _snapLogError   :: ByteString -> IO ()
-    , _snapSetTimeout :: Int -> IO () }
+    { _snapRequest       :: Request
+    , _snapResponse      :: Response
+    , _snapLogError      :: ByteString -> IO ()
+    , _snapModifyTimeout :: (Int -> Int) -> IO ()
+    }
 
 
 ------------------------------------------------------------------------------
@@ -181,12 +179,12 @@ snapFail !m = Snap $! return $! PassOnProcessing m
 
 ------------------------------------------------------------------------------
 instance MonadIO Snap where
-    liftIO m = Snap $ liftM SnapValue $ liftIO m
+    liftIO m = Snap $! liftM SnapValue $! liftIO m
 
 
 ------------------------------------------------------------------------------
 instance MonadCatchIO Snap where
-    catch (Snap m) handler = Snap $ m `catch` h
+    catch (Snap m) handler = Snap $! m `catch` h
       where
         h e = do
             rethrowIfUncatchable $ fromException e
@@ -208,10 +206,10 @@ rethrowIfUncatchable (Just e) = throw e
 
 ------------------------------------------------------------------------------
 instance MonadPlus Snap where
-    mzero = Snap $ return $ PassOnProcessing ""
+    mzero = Snap $! return $! PassOnProcessing ""
 
     a `mplus` b =
-        Snap $ do
+        Snap $! do
             r <- unSnap a
             -- redundant just in case ordering by frequency helps here.
             case r of
@@ -281,7 +279,7 @@ liftIter i = liftSnap $ Snap (lift i >>= return . SnapValue)
 -- immediately close the socket.
 runRequestBody :: MonadSnap m => Iteratee ByteString IO a -> m a
 runRequestBody iter = do
-    bumpTimeout <- liftM ($ 5) getTimeoutAction
+    bumpTimeout <- liftM ($ max 5) getTimeoutModifier
     req         <- getRequest
     senum       <- liftIO $ readIORef $ rqBody req
     let (SomeEnumerator enum) = senum
@@ -840,85 +838,20 @@ instance Show NoHandlerException where
 instance Exception NoHandlerException
 
 
-------------------------------------------------------------------------------
--- | An exception hierarchy for exceptions that cannot be caught by
--- user-defined error handlers
-data UncatchableException = forall e. Exception e => UncatchableException e
-  deriving (Typeable)
-
-
-------------------------------------------------------------------------------
-instance Show UncatchableException where
-    show (UncatchableException e) = "Uncatchable exception: " ++ show e
-
-
-------------------------------------------------------------------------------
-instance Exception UncatchableException
-
-
-------------------------------------------------------------------------------
-uncatchableExceptionToException :: Exception e => e -> SomeException
-uncatchableExceptionToException = toException . UncatchableException
-
-
-------------------------------------------------------------------------------
-uncatchableExceptionFromException :: Exception e => SomeException -> Maybe e
-uncatchableExceptionFromException e = do
-    UncatchableException ue <- fromException e
-    cast ue
-
-
-------------------------------------------------------------------------------
-data ConnectionTerminatedException =
-    ConnectionTerminatedException SomeException
-  deriving (Typeable)
-
-
-------------------------------------------------------------------------------
-instance Show ConnectionTerminatedException where
-    show (ConnectionTerminatedException e) =
-        "Connection terminated with exception: " ++ show e
-
-
-------------------------------------------------------------------------------
-instance Exception ConnectionTerminatedException where
-    toException   = uncatchableExceptionToException
-    fromException = uncatchableExceptionFromException
-
-
 ------------------------------------------------------------------------------
 -- | Terminate the HTTP session with the given exception.
 terminateConnection :: (Exception e, MonadCatchIO m) => e -> m a
 terminateConnection = throw . ConnectionTerminatedException . toException
 
 
--- | This is exception is thrown if the handler chooses to escape regular HTTP
--- traffic.
-data EscapeHttpException = EscapeHttpException
-    ((Int -> IO ()) -> Iteratee ByteString IO () -> Iteratee ByteString IO ())
-        deriving (Typeable)
-
-
-------------------------------------------------------------------------------
-instance Show EscapeHttpException where
-    show = const "HTTP traffic was escaped"
-
-
-------------------------------------------------------------------------------
-instance Exception EscapeHttpException where
-    toException   = uncatchableExceptionToException
-    fromException = uncatchableExceptionFromException
-
-
 ------------------------------------------------------------------------------
 -- | Terminate the HTTP session and hand control to some external handler,
 -- escaping all further HTTP traffic.
 --
--- The external handler takes two arguments: a function to tickle the timeout
--- manager, and a write end to the socket.
-escapeHttp :: MonadCatchIO m
-           => ((Int -> IO ()) -> Iteratee ByteString IO () ->
-                Iteratee ByteString IO ())
+-- The external handler takes two arguments: a function to modify the thread's
+-- timeout, and a write end to the socket.
+escapeHttp :: MonadCatchIO m =>
+              EscapeHttpHandler
            -> m ()
 escapeHttp = throw . EscapeHttpException
 
@@ -927,7 +860,7 @@ escapeHttp = throw . EscapeHttpException
 -- | Runs a 'Snap' monad action in the 'Iteratee IO' monad.
 runSnap :: Snap a
         -> (ByteString -> IO ())
-        -> (Int -> IO ())
+        -> ((Int -> Int) -> IO ())
         -> Request
         -> Iteratee ByteString IO (Request,Response)
 runSnap (Snap m) logerr timeoutAction req = do
@@ -941,11 +874,25 @@ runSnap (Snap m) logerr timeoutAction req = do
     return (_snapRequest ss', resp)
 
   where
-    fourohfour =
-        setContentLength 3 $
-        setResponseStatus 404 "Not Found" $
-        modifyResponseBody (>==> enumBuilder (fromByteString "404")) $
-        emptyResponse
+    fourohfour = do
+        clearContentLength                  $
+          setResponseStatus 404 "Not Found" $
+          setResponseBody enum404           $
+          emptyResponse
+
+    enum404 = enumBuilder $ mconcat $ map fromByteString html
+
+    html = [ S.concat [ "<!DOCTYPE html>\n"
+                      , "<html>\n"
+                      , "<head>\n"
+                      , "<title>Not found</title>\n"
+                      , "</head>\n"
+                      , "<body>\n"
+                      , "<code>No handler accepted \""
+                      ]
+           , rqURI req
+           , "\"</code>\n</body></html>"
+           ]
 
     dresp = emptyResponse { rspHttpVersion = rqVersion req }
 
@@ -956,7 +903,7 @@ runSnap (Snap m) logerr timeoutAction req = do
 ------------------------------------------------------------------------------
 evalSnap :: Snap a
          -> (ByteString -> IO ())
-         -> (Int -> IO ())
+         -> ((Int -> Int) -> IO ())
          -> Request
          -> Iteratee ByteString IO a
 evalSnap (Snap m) logerr timeoutAction req = do
@@ -973,6 +920,16 @@ evalSnap (Snap m) logerr timeoutAction req = do
 {-# INLINE evalSnap #-}
 
 
+------------------------------------------------------------------------------
+getParamFrom :: MonadSnap m =>
+                (ByteString -> Request -> Maybe [ByteString])
+             -> ByteString
+             -> m (Maybe ByteString)
+getParamFrom f k = do
+    rq <- getRequest
+    return $! liftM (S.intercalate " ") $ f k rq
+{-# INLINE getParamFrom #-}
+
 
 ------------------------------------------------------------------------------
 -- | See 'rqParam'. Looks up a value for the given named parameter in the
@@ -984,9 +941,38 @@ evalSnap (Snap m) logerr timeoutAction req = do
 getParam :: MonadSnap m
          => ByteString          -- ^ parameter name to look up
          -> m (Maybe ByteString)
-getParam k = do
-    rq <- getRequest
-    return $ liftM (S.intercalate " ") $ rqParam k rq
+getParam = getParamFrom rqParam
+{-# INLINE getParam #-}
+
+
+------------------------------------------------------------------------------
+-- | See 'rqPostParam'. Looks up a value for the given named parameter in the
+-- POST form parameters mapping in 'Request'. If more than one value was
+-- entered for the given parameter name, 'getPostParam' gloms the values
+-- together with:
+--
+-- @    'S.intercalate' \" \"@
+--
+getPostParam :: MonadSnap m
+             => ByteString          -- ^ parameter name to look up
+             -> m (Maybe ByteString)
+getPostParam = getParamFrom rqPostParam
+{-# INLINE getPostParam #-}
+
+
+------------------------------------------------------------------------------
+-- | See 'rqQueryParam'. Looks up a value for the given named parameter in the
+-- query string parameters mapping in 'Request'. If more than one value was
+-- entered for the given parameter name, 'getQueryParam' gloms the values
+-- together with:
+--
+-- @    'S.intercalate' \" \"@
+--
+getQueryParam :: MonadSnap m
+              => ByteString          -- ^ parameter name to look up
+              -> m (Maybe ByteString)
+getQueryParam = getParamFrom rqQueryParam
+{-# INLINE getQueryParam #-}
 
 
 ------------------------------------------------------------------------------
@@ -996,6 +982,20 @@ getParams :: MonadSnap m => m Params
 getParams = getRequest >>= return . rqParams
 
 
+------------------------------------------------------------------------------
+-- | See 'rqParams'. Convenience function to return 'Params' from the
+-- 'Request' inside of a 'MonadSnap' instance.
+getPostParams :: MonadSnap m => m Params
+getPostParams = getRequest >>= return . rqPostParams
+
+
+------------------------------------------------------------------------------
+-- | See 'rqParams'. Convenience function to return 'Params' from the
+-- 'Request' inside of a 'MonadSnap' instance.
+getQueryParams :: MonadSnap m => m Params
+getQueryParams = getRequest >>= return . rqQueryParams
+
+
 ------------------------------------------------------------------------------
 -- | Gets the HTTP 'Cookie' with the specified name.
 getCookie :: MonadSnap m
@@ -1030,15 +1030,36 @@ expireCookie nm dm = do
 
 ------------------------------------------------------------------------------
 -- | Causes the handler thread to be killed @n@ seconds from now.
-setTimeout :: MonadSnap m
-           => Int -> m ()
-setTimeout n = do
-    t <- getTimeoutAction
-    liftIO $ t n
+setTimeout :: MonadSnap m => Int -> m ()
+setTimeout = modifyTimeout . const
 
 
 ------------------------------------------------------------------------------
--- | Returns an 'IO' action which you can use to reset the handling thread's
+-- | Causes the handler thread to be killed at least @n@ seconds from now.
+extendTimeout :: MonadSnap m => Int -> m ()
+extendTimeout = modifyTimeout . max
+
+
+------------------------------------------------------------------------------
+-- | Modifies the amount of time remaining before the request times out.
+modifyTimeout :: MonadSnap m => (Int -> Int) -> m ()
+modifyTimeout f = do
+    m <- getTimeoutModifier
+    liftIO $ m f
+
+
+------------------------------------------------------------------------------
+-- | Returns an 'IO' action which you can use to set the handling thread's
 -- timeout value.
 getTimeoutAction :: MonadSnap m => m (Int -> IO ())
-getTimeoutAction = liftSnap $ liftM _snapSetTimeout sget
+getTimeoutAction = do
+    modifier <- liftSnap $ liftM _snapModifyTimeout sget
+    return $! modifier . const
+{-# DEPRECATED getTimeoutAction
+      "use getTimeoutModifier instead. Since 0.8." #-}
+
+
+------------------------------------------------------------------------------
+-- | Returns an 'IO' action which you can use to modify the timeout value.
+getTimeoutModifier :: MonadSnap m => m ((Int -> Int) -> IO ())
+getTimeoutModifier = liftSnap $ liftM _snapModifyTimeout sget
