@@ -74,7 +74,7 @@ getSafePath = do
     let dirs = splitDirectories p
     when (elem ".." dirs) pass
 
-    return $ joinPath dirs
+    return $! joinPath dirs
 
 
 ------------------------------------------------------------------------------
@@ -110,6 +110,7 @@ type MimeMap = Map FilePath ByteString
 -- >   ( ".hs"      , "text/plain"                        ),
 -- >   ( ".htm"     , "text/html"                         ),
 -- >   ( ".html"    , "text/html"                         ),
+-- >   ( ".ico"     , "image/x-icon"                      ),
 -- >   ( ".jar"     , "application/x-java-archive"        ),
 -- >   ( ".jpeg"    , "image/jpeg"                        ),
 -- >   ( ".jpg"     , "image/jpeg"                        ),
@@ -169,6 +170,7 @@ defaultMimeTypes = Map.fromList [
   ( ".hs"      , "text/plain"                        ),
   ( ".htm"     , "text/html"                         ),
   ( ".html"    , "text/html"                         ),
+  ( ".ico"     , "image/x-icon"                      ),
   ( ".jar"     , "application/x-java-archive"        ),
   ( ".jpeg"    , "image/jpeg"                        ),
   ( ".jpg"     , "image/jpeg"                        ),
@@ -324,7 +326,7 @@ defaultIndexGenerator mm styles d = do
         f <- liftIO $ liftM (\s -> T.encodeUtf8 s `mappend` "/") $ packFn f0
         writeBS "<tr><td class='filename'><a href='"
         writeBS f
-        writeBS "/'>"
+        writeBS "'>"
         writeBS f
         writeBS "</a></td><td class='type' colspan=2>DIR</td></tr>"
 
@@ -607,7 +609,7 @@ rangeParser = string "bytes=" *>
         char '-'
         end   <- option Nothing $ liftM Just parseNum
 
-        return $ RangeReq start end
+        return $! RangeReq start end
 
     suffixByteRangeSpec = liftM SuffixRangeReq $ char '-' *> parseNum
 
