@@ -252,7 +252,7 @@ handleMultipart uploadPolicy origPartHandler = do
     hdrs <- liftM headers getRequest
     let (ct, mbBoundary) = getContentType hdrs
 
-    tickleTimeout <- getTimeoutAction
+    tickleTimeout <- liftM (. max) getTimeoutModifier
     let bumpTimeout = tickleTimeout $ uploadTimeout uploadPolicy
 
     let partHandler = if doProcessFormInputs uploadPolicy
@@ -274,7 +274,11 @@ handleMultipart uploadPolicy origPartHandler = do
     let boundary = fromJust mbBoundary
     captures <- runRequestBody (iter bumpTimeout boundary partHandler)
 
-    procCaptures [] captures
+    xs <- procCaptures [] captures
+    modifyRequest $ \req ->
+        let pp = rqPostParams req
+        in rqModifyParams (\p -> Map.unionWith (++) p pp) req
+    return xs
 
   where
     rateLimit bump m =
@@ -297,16 +301,18 @@ handleMultipart uploadPolicy origPartHandler = do
 
     maxFormVars = maximumNumberOfFormInputs uploadPolicy
 
-    procCaptures l [] = return $ reverse l
+    modifyParams f r = r { rqPostParams = f $ rqPostParams r }
+
+    procCaptures l [] = return $! reverse l
     procCaptures l ((File x):xs) = procCaptures (x:l) xs
     procCaptures l ((Capture k v):xs) = do
         rq <- getRequest
-        let n = Map.size $ rqParams rq
+        let n = Map.size $ rqPostParams rq
         when (n >= maxFormVars) $
           throw $ PolicyViolationException $
           T.concat [ "number of form inputs exceeded maximum of "
                    , T.pack $ show maxFormVars ]
-        modifyRequest $ rqModifyParams (ins k v)
+        modifyRequest $ modifyParams (ins k v)
         procCaptures l xs
 
 
@@ -565,7 +571,7 @@ captureVariableOrReadFile maxSize fileHandler partInfo =
         var <- liftM S.concat $
                joinI' $
                takeNoMoreThan maxSize $$ consume
-        return $ Capture fieldName var
+        return $! Capture fieldName var
 
     handler e = do
         debug $ "captureVariableOrReadFile/handler: caught " ++ show e
@@ -781,15 +787,15 @@ processParts partIter = iterateeDebugWrapper "processParts" $ go D.empty
         if isLast
           then return Nothing
           else do
-            x <- partIter
+            !x <- partIter
             skipToEof
-            return $ Just x
+            return $! Just x
 
-    go soFar = {-# SCC "processParts/go" #-} do
+    go !soFar = {-# SCC "processParts/go" #-} do
       b <- isEOF
 
       if b
-        then return $ D.toList soFar
+        then return $! D.toList soFar
         else do
            -- processPart $$ iter
            --   :: Iteratee MatchInfo m (Step ByteString m a)
@@ -800,7 +806,7 @@ processParts partIter = iterateeDebugWrapper "processParts" $ go D.empty
 
            case output of
              Just x  -> go (D.append soFar $ D.singleton x)
-             Nothing -> return $ D.toList soFar
+             Nothing -> return $! D.toList soFar
 
     bParser = iterateeDebugWrapper "boundary debugger" $
                   iterParser $ pBoundaryEnd
