@@ -30,7 +30,8 @@ import           Test.Framework
 import           Test.Framework.Providers.HUnit
 import           Test.Framework.Providers.QuickCheck2
 import           Test.HUnit hiding (Test, path)
-                 
+
+import           Snap.Internal.Exceptions
 import           Snap.Internal.Http.Types
 import           Snap.Internal.Parsing
 import           Snap.Internal.Types
@@ -96,9 +97,10 @@ mkRequest :: ByteString -> IO Request
 mkRequest uri = do
     enum <- newIORef $ SomeEnumerator returnI
 
-    return $ Request "foo" 80 "127.0.0.1" 999 "foo" 1000 "foo" False H.empty
-                     enum Nothing GET (1,1) [] "" uri "/"
-                     (S.concat ["/",uri]) "" Map.empty
+    return $! Request "foo" 80 "127.0.0.1" 999 "foo" 1000 "foo" False H.empty
+                      enum Nothing GET (1,1) [] uri "/"
+                      (S.concat ["/",uri]) "" Map.empty Map.empty Map.empty
+
 
 mkRequestQuery :: ByteString -> ByteString -> [ByteString] -> IO Request
 mkRequestQuery uri k v = do
@@ -108,8 +110,8 @@ mkRequestQuery uri k v = do
     let q  = S.concat [k,"=", S.concat v]
 
     return $ Request "foo" 80 "foo" 999 "foo" 1000 "foo" False H.empty
-                     enum Nothing GET (1,1) [] "" uri "/"
-                     (S.concat ["/",uri,"?",q]) q mp
+                     enum Nothing GET (1,1) [] uri "/"
+                     (S.concat ["/",uri,"?",q]) q mp mp Map.empty
 
 
 mkZomgRq :: IO Request
@@ -117,7 +119,8 @@ mkZomgRq = do
     enum <- newIORef $ SomeEnumerator returnI
 
     return $ Request "foo" 80 "127.0.0.1" 999 "foo" 1000 "foo" False H.empty
-                     enum Nothing GET (1,1) [] "" "/" "/" "/" "" Map.empty
+                     enum Nothing GET (1,1) [] "/" "/" "/" ""
+                     Map.empty Map.empty Map.empty
 
 
 mkIpHeaderRq :: IO Request
@@ -136,8 +139,8 @@ mkRqWithEnum :: (forall a . Enumerator ByteString IO a) -> IO Request
 mkRqWithEnum e = do
     enum <- newIORef $ SomeEnumerator e
     return $ Request "foo" 80 "foo" 999 "foo" 1000 "foo" False H.empty
-                 enum Nothing GET (1,1) [] "" "/" "/" "/" ""
-                 Map.empty
+                 enum Nothing GET (1,1) [] "/" "/" "/" ""
+                 Map.empty Map.empty Map.empty
 
 testCatchIO :: Test
 testCatchIO = testCase "types/catchIO" $ do
@@ -160,21 +163,21 @@ testCatchIO = testCase "types/catchIO" $ do
 go :: Snap a -> IO (Request,Response)
 go m = do
     zomgRq <- mkZomgRq
-    run_ $ runSnap m dummy dummy zomgRq
+    run_ $ runSnap m dummy (const (return ())) zomgRq
   where
     dummy !x = return $! (show x `using` rdeepseq) `seq` ()
 
 goIP :: Snap a -> IO (Request,Response)
 goIP m = do
     rq <- mkIpHeaderRq
-    run_ $ runSnap m dummy dummy rq
+    run_ $ runSnap m dummy (const (return ())) rq
   where
     dummy = const $ return ()
 
 goPath :: ByteString -> Snap a -> IO (Request,Response)
 goPath s m = do
     rq <- mkRequest s
-    run_ $ runSnap m dummy dummy rq
+    run_ $ runSnap m dummy (const (return ())) rq
   where
     dummy = const $ return ()
 
@@ -186,7 +189,7 @@ goPathQuery :: ByteString
             -> IO (Request,Response)
 goPathQuery s k v m = do
     rq <- mkRequestQuery s k v
-    run_ $ runSnap m dummy dummy rq
+    run_ $ runSnap m dummy (const (return ())) rq
   where
     dummy = const $ return ()
 
@@ -194,7 +197,7 @@ goPathQuery s k v m = do
 goBody :: Snap a -> IO (Request,Response)
 goBody m = do
     rq <- mkRqWithBody
-    run_ $ runSnap m dummy dummy rq
+    run_ $ runSnap m dummy (const (return ())) rq
   where
     dummy = const $ return ()
 
@@ -204,7 +207,7 @@ goEnum :: (forall a . Enumerator ByteString IO a)
        -> IO (Request,Response)
 goEnum enum m = do
     rq <- mkRqWithEnum enum
-    run_ $ runSnap m dummy dummy rq
+    run_ $ runSnap m dummy (const (return ())) rq
   where
     dummy = const $ return ()
 
@@ -284,7 +287,7 @@ testBracketSnap = testCase "types/bracketSnap" $ do
                         (const $ return ())
                         (const $ return ())
                         rq
-                         
+
     y' <- readIORef ref
     assertEqual "bracketSnap/after" 2 y'
 
@@ -294,7 +297,7 @@ testBracketSnap = testCase "types/bracketSnap" $ do
                         (const $ return ())
                         (const $ return ())
                         rq
-                         
+
     y'' <- readIORef ref
     assertEqual "bracketSnap/after" 3 y''
 
@@ -473,13 +476,20 @@ testParam :: Test
 testParam = testCase "types/getParam" $ do
     expect404 $ goPath "/foo" f
     expectNo404 $ goPathQuery "/foo" "param" ["foo"] f
+    expectNo404 $ goPathQuery "/foo" "param" ["foo"] fQ
+    expect404 $ goPathQuery "/foo" "param" ["foo"] fP
+
   where
-    f = do
-        mp <- getParam "param"
+    p gp = do
+        mp <- gp "param"
         maybe pass
               (\s -> if s == "foo" then return () else pass)
               mp
 
+    f  = p getParam
+    fQ = p getQueryParam
+    fP = p getPostParam
+
 
 getBody :: Response -> IO L.ByteString
 getBody r = do
@@ -546,10 +556,8 @@ testIpHeaderFilter = testCase "types/ipHeaderFilter" $ do
 testMZero404 :: Test
 testMZero404 = testCase "types/mzero404" $ do
     (_,r) <- go mzero
-    let l = rspContentLength r
     b <- getBody r
-    assertEqual "mzero 404" "404" b
-    assertEqual "mzero 404 length" (Just 3) l
+    assertBool "mzero 404" ("<!DOCTYPE html" `L.isPrefixOf` b)
 
 
 testEvalSnap :: Test
